local addonName, addonTable = ...
local ldb = LibStub:GetLibrary("LibDataBroker-1.1", true)
local icon = ldb and LibStub("LibDBIcon-1.0",true)
local WWLDB = nil
local C_Timer = C_Timer
local C_Garrison = C_Garrison
local TAKE_ACTION_NOTHING, TAKE_ACTION_WORK, TAKE_ACTION_SLEEP = 0, 1, 2

local function dbg(text)
  --[===[@debug@
  DEFAULT_CHAT_FRAME:AddMessage("|cff2476ffDebugWorkWork:|r " .. text)
  --@end-debug@]===]
end

local function say(text)
  DEFAULT_CHAT_FRAME:AddMessage("|cff2476ffWorkWork:|r " .. text)
end


local WorkWork = CreateFrame("Frame")
WorkWork.Events = {}
WorkWork.StopAllTimers = nil
WorkWork:RegisterEvent("ADDON_LOADED")
WorkWork:RegisterEvent("GARRISON_ARCHITECT_OPENED")
WorkWork:RegisterEvent("GARRISON_ARCHITECT_CLOSED")
WorkWork:SetScript("OnEvent",function( self, event, ...) 
  self.Events[event](self, ...)
end)
  if WorkWork_TakeAction == TAKE_ACTION_NOTHING then
    WWLDB.icon = "Interface\\Icons\\inv_misc_questionmark"
    WWLDB.label = "Stopped"
  elseif WorkWork_TakeAction == TAKE_ACTION_WORK then
    WWLDB.icon = "Interface\\Icons\\inv_mace_10"
    WWLDB.label = "WorkWork"
  elseif WorkWork_TakeAction == TAKE_ACTION_SLEEP then
    WorkWork:RegisterEvent("GARRISON_ARCHITECT_OPENED")
    WWLDB.icon = "Interface\\Icons\\ability_hibernation"
    WWLDB.label = "zZzZZ"
  end
WorkWork.Events["ADDON_LOADED"] = function( self, loadedAddon ) 
  if loadedAddon ~= addonName then return end
  self:UnregisterEvent("ADDON_LOADED")
  WorkWork_TakeAction = WorkWork_TakeAction or TAKE_ACTION_WORK
  
  -- LDB 
  WWLDB = ldb:NewDataObject("WorkWork", {
    type = "launcher",
    label = "WorkWork",
    icon = "Interface\\Icons\\inv_mace_10",
    OnClick = function(self, button)
      WorkWork:CycleStates()
    end,
  })
  
  -- Update icon and text to the current state
  self:UpdateLDB()
end

WorkWork.Events["GARRISON_ARCHITECT_OPENED"] = function( self )
  self.StopAllTimers = nil
  C_Timer.After( 0.1, function() WorkWork:TimerCallback() end )
end

WorkWork.TimerCallback = function( self )
  if WorkWork_TakeAction == TAKE_ACTION_WORK then
    -- Smack those peons and get them to work
    self:SmackPeons()
  elseif WorkWork_TakeAction == TAKE_ACTION_SLEEP then
    -- Sing a lullaby, making peons sleepy and leaving work
    self:SingLullaby()
  end  
end

WorkWork.Events["GARRISON_ARCHITECT_CLOSED"] = function( self ) 
  self.StopAllTimers = true
end

WorkWork.buildings = {}
WorkWork.followers = {}
WorkWork.followersList = {}

-- C_C_Garrison.AssignFollowerToBuilding(GarrisonBuildingFrame.selectedBuilding.plotID, self.info.followerID);
-- C_C_Garrison.GetBuildings() to get a table of BuildingID <=> PlotID
WorkWork.SortFollowers = GarrisonFollowerList_SortFollowers
function WorkWork:SmackPeons()
  self.buildings = C_Garrison.GetBuildings()
  local are_we_done = true
  
  -- Got buildings to work with
  if #self.buildings > 0 then
    for i = 1, #self.buildings do
      -- Only search the followers if the building is not occupied yet!
      if not C_Garrison.GetFollowerInfoForBuilding(self.buildings[i].plotID) then
        -- Get a list of ALL follower who can work, includes inactives and ones currently on missions!
        self.followers = C_Garrison.GetPossibleFollowersForBuilding(self.buildings[i].plotID);
        if (#self.followers > 0) then
          -- Sorts the followers like wow would from best to worst
          self.SortFollowers(self)
   
          for j = 1, #self.followers do
            -- Use the first one thats available
            -- C_Garrison.GetFollowerStatus returns nil for available
            if not C_Garrison.GetFollowerStatus(self.followers[j].followerID) then
              dbg(self.followers[j].name .. " (" ..self.followers[j].followerID..") => " ..self.buildings[i].texPrefix .." (" ..self.buildings[i].plotID..")")
              self:GoToWork(self.buildings[i].plotID, self.followers[j].followerID)
              are_we_done = false
            end -- C_Garrison.GetFollowerStatus
          end -- for followers
        end -- followers > 0
      end -- someone already working?
    end -- for buildings

    if are_we_done == true then
      -- All possible work slots filled
      self:UpdateLDB( true )
      dbg("Smacked all the peons")
    else
      -- We tried to assign at least one worker, as long as the architect table was not closed in the meantime go for a nother run in 0.1s
      if not self.StopAllTimers then 
        C_Timer.After( 0.1, function() WorkWork:TimerCallback() end )
      end
    end
  end
end

function WorkWork:GoToWork( plot, follower )
  C_Garrison.AssignFollowerToBuilding( plot, follower ) 
end


-- To clear all buildings of their followers we just need to iterate over all the buildings.
function WorkWork:SingLullaby()
  self.buildings = C_Garrison.GetBuildings()
  local are_we_done = true
  if #self.buildings > 0 then
    for i = 1, #self.buildings do
      if C_Garrison.GetFollowerInfoForBuilding( self.buildings[i].plotID ) then
        self:GoToSleep( self.buildings[i].plotID )
        are_we_done = false
      end
    end -- for buildings
  end -- buildings > 0


  if are_we_done == true then 
    self:UpdateLDB( true )
    dbg("Lullaby sung, all peons recalled from their workplaces")
  else
    if not self.StopAllTimers then 
      C_Timer.After( 0.1, function() WorkWork:TimerCallback() end )
    end
  end
end

function WorkWork:GoToSleep( plot )
  C_Garrison.RemoveFollowerFromBuilding( plot ) 
end

SLASH_WORKWORK1, SLASH_WORKWORK2 = '/workwork', '/ww';

local STATES = {
  [TAKE_ACTION_NOTHING] = "No actions taken",
  [TAKE_ACTION_WORK]    = "Smacking peons",
  [TAKE_ACTION_SLEEP]   = "Singing Lullaby",
}
function WorkWork:CycleStates()
  WorkWork_TakeAction = ( WorkWork_TakeAction + 1 ) % 3

  -- Do nothing? => Unregister event
  -- Do something? => Register it again
  self:UpdateLDB()
  dbg(STATES[WorkWork_TakeAction])
  return WorkWork_TakeAction
end

function WorkWork:UpdateLDB( done )
  if WorkWork_TakeAction == TAKE_ACTION_NOTHING then
    self:UnregisterEvent("GARRISON_ARCHITECT_OPENED")
    --WWLDB.icon = "Interface\\Icons\\inv_misc_questionmark"
    WWLDB.label = "Stopped" 
  elseif WorkWork_TakeAction == TAKE_ACTION_WORK then
    self:RegisterEvent("GARRISON_ARCHITECT_OPENED")
    --WWLDB.icon = "Interface\\Icons\\inv_mace_10"
    WWLDB.label = "WorkWork" .. ( done and "(done)" or "" )
  elseif WorkWork_TakeAction == TAKE_ACTION_SLEEP then
    self:RegisterEvent("GARRISON_ARCHITECT_OPENED")
    --WWLDB.icon = "Interface\\Icons\\ability_hibernation"
    WWLDB.label = "zZzZZ" .. ( done and "(done)" or "" )
  else
    dbg("This should not happen!")
    WorkWork_TakeAction = TAKE_ACTION_NOTHING
  end
end
local function handler(msg, editbox)
  WorkWork:CycleStates()
end
SlashCmdList["WORKWORK"] = handler;

dbg("loaded")